@@ -178,7 +178,7 @@ int MADB_KeyTypeCount(MADB_Dbc *Connection, char *TableName, int *PrimaryKeysCou
   }
   p+= _snprintf(p, sizeof(StmtStr) - (p - StmtStr), "%s LIMIT 0", TableName);
   LOCK_MARIADB(Connection);
-  if (SQL_SUCCEEDED(MADB_RealQuery(Connection, StmtStr, (unsigned long)(p - StmtStr), &Connection->Error)) &&
+  if (SQL_SUCCEEDED(MADB_RealQuery(Connection, StmtStr, (p - StmtStr), &Connection->Error)) &&
      (Res= mysql_store_result(Connection->mariadb)) != NULL)
   {
     Count= mysql_field_count(Connection->mariadb);
@@ -364,7 +364,7 @@ MYSQL_RES *MADB_GetDefaultColumnValues(MADB_Stmt *Stmt, MYSQL_FIELD *fields)
     goto error;
 
   LOCK_MARIADB(Stmt->Connection);
-  if (SQL_SUCCEEDED(MADB_RealQuery(Stmt->Connection, DynStr.str, (SQLINTEGER)DynStr.length, &Stmt->Error)))
+  if (SQL_SUCCEEDED(MADB_RealQuery(Stmt->Connection, DynStr.str, DynStr.length, &Stmt->Error)))
   {
     result= mysql_store_result(Stmt->Connection->mariadb);
   }
@@ -440,7 +440,7 @@ SQLULEN MADB_GetDataSize(SQLSMALLINT SqlType, unsigned long long OctetLength, BO
 }
 
 /* {{{ MADB_GetDisplaySize */
-size_t MADB_GetDisplaySize(MYSQL_FIELD *Field, MARIADB_CHARSET_INFO *charset)
+size_t MADB_GetDisplaySize(MYSQL_FIELD *Field, MARIADB_CHARSET_INFO *charset, BOOL noBigint)
 {
   /* Todo: check these values with output from mysql --with-columntype-info */
   switch (Field->type) {
@@ -458,7 +458,7 @@ size_t MADB_GetDisplaySize(MYSQL_FIELD *Field, MARIADB_CHARSET_INFO *charset)
   case MYSQL_TYPE_LONG:
     return 11 - test(Field->flags & UNSIGNED_FLAG);
   case MYSQL_TYPE_LONGLONG:
-    return 20;
+    return (noBigint ? 11 - test(Field->flags & UNSIGNED_FLAG) : 20);
   case MYSQL_TYPE_DOUBLE:
     return 15;
   case MYSQL_TYPE_FLOAT:
@@ -565,7 +565,8 @@ size_t MADB_GetOctetLength(MYSQL_FIELD *Field, unsigned short MaxCharLen)
 }
 /* }}} */
 
-/* {{{ MADB_GetDefaultType */
+/* {{{ MADB_GetDefaultType
+ */
 int MADB_GetDefaultType(int SQLDataType)
 {
   switch(SQLDataType)
@@ -621,9 +622,10 @@ int MADB_GetDefaultType(int SQLDataType)
 }
 /* }}} */
 
-/* {{{ MapMariadDbToOdbcType */
-       /* It's not quite right to mix here C and SQL types, even though constants are sort of equal */
-SQLSMALLINT MapMariadDbToOdbcType(MYSQL_FIELD *field)
+/* {{{ MapMariadDbToOdbcType
+ * @param Stmt - we need it for the information that may need to do some ajustments in the mappings 
+ */
+SQLSMALLINT MapMariadDbToOdbcType(MADB_Stmt* Stmt, MYSQL_FIELD *field)
 {
   switch (field->type) {
     case MYSQL_TYPE_BIT:
@@ -658,7 +660,15 @@ SQLSMALLINT MapMariadDbToOdbcType(MYSQL_FIELD *field)
     case MYSQL_TYPE_LONG_BLOB:
       return MADB_FIELD_IS_BINARY(field) ? SQL_LONGVARBINARY : SQL_LONGVARCHAR;
     case MYSQL_TYPE_LONGLONG:
-      return SQL_BIGINT;
+      /* Treating MSAccess differently */
+      if (Stmt->Connection->Dsn->NoBigint)
+      {
+        return SQL_INTEGER;
+      }
+      else
+      {
+        return Stmt->Connection->Environment->AppType == ATypeMSAccess ? SQL_VARCHAR : SQL_BIGINT;
+      }
     case MYSQL_TYPE_STRING:
       return MADB_FIELD_IS_BINARY(field) ? SQL_BINARY : SQL_CHAR;
     case MYSQL_TYPE_VAR_STRING:
@@ -750,6 +760,7 @@ int MADB_GetMaDBTypeAndLength(SQLINTEGER SqlDataType, my_bool *Unsigned, unsigne
     return MYSQL_TYPE_LONG;
   case SQL_C_UBIGINT:
   case SQL_C_SBIGINT:
+  case SQL_BIGINT:
     *Length= sizeof(long long);
     *Unsigned= (SqlDataType == SQL_C_UBIGINT);
     return MYSQL_TYPE_LONGLONG;
@@ -837,6 +848,11 @@ SQLRETURN MADB_CopyMadbTimestamp(MADB_Stmt *Stmt, MYSQL_TIME *tm, SQLPOINTER Dat
           ts->month= cur_tm->tm_mon + 1;
           ts->day= cur_tm->tm_mday;
           ts->fraction= 0;
+          /* If we allowing fraction in time on the way to the server, we should allow it on the way back
+           */
+          if (Stmt->Connection->Dsn->AllowDtTruncation) {
+            ts->fraction= tm->second_part * 1000;
+          }
         }
         else
         {
@@ -1303,18 +1319,23 @@ int MADB_FindNextDaeParam(MADB_Desc *Desc, int InitialParam, SQLSMALLINT RowNumb
   return MADB_NOPARAM;
 }
 
-
-BOOL MADB_IsNumericType(SQLSMALLINT ConciseType)
+/* Returns true if the type is */
+BOOL MADB_IsDecimalType(SQLSMALLINT ConciseType)
 {
   switch (ConciseType)
   {
     case SQL_C_DOUBLE:
     case SQL_C_FLOAT:
     case SQL_DECIMAL:
+    case SQL_C_NUMERIC:
       return TRUE;
   }
+  return FALSE;
+}
 
-  return MADB_IsIntType(ConciseType);
+BOOL MADB_IsNumericType(SQLSMALLINT ConciseType)
+{
+  return MADB_IsDecimalType(ConciseType) || MADB_IsIntType(ConciseType);
 }
 
 
@@ -1355,4 +1376,35 @@ void MADB_InstallStmt(MADB_Stmt *Stmt, MYSQL_STMT *stmt)
     MADB_StmtResetResultStructures(Stmt);
     MADB_DescSetIrdMetadata(Stmt, mysql_fetch_fields(FetchMetadata(Stmt)), mysql_stmt_field_count(Stmt->stmt));
   }
+}
+
+
+enum enum_field_types MADB_GetNativeFieldType(MADB_Stmt *Stmt, int i)
+{
+  /* Assuming the caller knows what is it calling for - i.e. i is valid, and stmt->metadata is set. Should be */
+  return mysql_fetch_fields(Stmt->metadata)[i].type;
+}
+
+
+SQLRETURN MADB_KillAtServer(MADB_Stmt *Stmt)
+{
+  MYSQL *MariaDb, *Kill=Stmt->Connection->mariadb;
+
+  if (!(MariaDb= mysql_init(NULL)))
+  {
+    return MADB_SetError(&Stmt->Error, MADB_ERR_HY001, NULL, 0);
+  }
+  if (SQL_SUCCEEDED(MADB_DbcCoreConnect(Stmt->Connection, MariaDb, Stmt->Connection->Dsn, &Stmt->Error, 0)))
+  {
+    char StmtStr[32];
+    // 32 is enough to fit the query with longest 8byte unsigned  thread id, thus sprintf is fine here
+    int len= sprintf(StmtStr, "KILL QUERY %lu", mysql_thread_id(Kill));
+    if (mysql_real_query(MariaDb, StmtStr, len))
+    {
+      mysql_close(MariaDb);
+      return MADB_SetError(&Stmt->Error, MADB_ERR_HY000, "Error while terminating the process on the server", 0);
+    }
+  }
+  mysql_close(MariaDb);
+  return SQL_SUCCESS;
 }
\ No newline at end of file
